/***************************************************************************
 *            gametree.c
 *
 *  Wed Jul 14 17:52:15 2004
 *  Copyright  2004  Mithun Chaubey
 *  mithun@mithunc.net
 ****************************************************************************/

/*
 *  This file is a part of Grapevine.
 *
 *  Grapevine is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>

#include <gtk/gtk.h>
#include <glade/glade.h>

#include "config.h"

#include "prefs.h"
#include "desktop_entry.h"
#include "gametree.h"

uint MAX_VALUE_LENGTH = 500;

// Recursive function to read game and directory information into a treestore
void FillStore(GtkTreeStore* tree, const char* path, GtkTreeIter* parent);

// Add P2P games to tree
void AddP2PEntries(GtkTreeStore* gametree);

void LoadIcon(GtkTreeStore* tree, GtkTreeIter* iterptr, const char* iconpath)
{
	GdkPixbuf* icon;
	GError* pixbuferror = NULL;
	
	icon = gdk_pixbuf_new_from_file(iconpath, &pixbuferror);
					
	if(icon == NULL){
		printf("Error loading pixbuf: %s\n", pixbuferror->message);
	
	} else {
		// enter the icon
		gtk_tree_store_set(
			tree, iterptr,
			ICON_COLUMN, icon,
			-1);
	}
}

// Recursive function to read game and directory information into a treestore
void FillStore(GtkTreeStore* tree, const char* path, GtkTreeIter* parent)
{   
    DIR* Dir = opendir(path);
    if(Dir == NULL){
    printf("Error opening directory: %s\n", path);
	    return;
	}
    
    struct dirent* entry;
    entry = readdir(Dir);
    
    // this will traverse the current level of folders
	GtkTreeIter iter;
	
	// these are for traversing the next level of folders
	GtkTreeIter nestIter;
	char* nestpath;
	    
	// these will be needed to get the properties from the desktop entry
	char* shortcutpath;
	char* shortcutname;
	char* iconpath;
	
	// some information on the file will be stored in this
	struct stat fileinfo;
			
    while(entry != NULL){
		
		// this is the best way I could think of handling the . and .. entries, for now
		while(entry != NULL){
			if(entry->d_name[0] == '.') 
				entry = readdir(Dir);
			else
				break;
		}
		if(entry == NULL){
			closedir(Dir);
			return;
		}
		
		// build the path for this directory
		nestpath = malloc(strlen(path) + strlen(entry->d_name) + 2);
		sprintf(nestpath, "%s/%s", path, entry->d_name);
		
		stat(nestpath, &fileinfo);
		
		// if(entry->d_type == DT_DIR)
		if(S_ISDIR(fileinfo.st_mode))
        {		
			//add a row and put the directory there
			gtk_tree_store_append (tree, &iter, parent);
			gtk_tree_store_set(
				tree, &iter, 
				NAME_COLUMN, entry->d_name,
				PATH_COLUMN, nestpath,
				TYPE_COLUMN, DIR_TYPE,
				-1);
                               
            // call this function recursively to traverse this directory
			// note that the current node is being called as the parent
			nestIter = iter;
            FillStore(tree, nestpath, &nestIter);
			free(nestpath);
        } // if DT_DIR
			
        else if(S_ISREG(fileinfo.st_mode) && 
            strstr(entry->d_name, ".desktop") != NULL )
        {
            // build the path to the shortcut
			shortcutpath = malloc(strlen(path) + strlen(entry->d_name) + 2);
			sprintf(shortcutpath, "%s/%s", path, entry->d_name);
			
			// get the name and icon path from the shortcut
			shortcutname = desktop_entry_GetValue("Name", shortcutpath);
			iconpath =  desktop_entry_GetValue("Icon", shortcutpath);
               
			// add a row to the tree
			if(shortcutname != NULL)
			{
				gtk_tree_store_append (tree, &iter, parent);
			
            	// store the icon
				if(iconpath != NULL)
					LoadIcon(tree, &iter, iconpath);
				
				// store the other fields
				gtk_tree_store_set(
						tree, &iter,
						NAME_COLUMN, shortcutname,
						PATH_COLUMN, shortcutpath,
						TYPE_COLUMN, SHORTCUT_TYPE,
						-1);
			
				free(shortcutname);
				free(iconpath);
				free(shortcutpath);
			} // if name != NULL
		} // if DT_REG
		
        entry = readdir(Dir);    
    } // while
    
	closedir(Dir);
 }

void AddP2PEntries(GtkTreeStore* gametree)
{
	// path = path to ~/.point2play
	char* path = malloc( strlen(getenv("HOME")) + 12 + 1);
	sprintf(path, "%s%s", getenv("HOME"), "/.point2play");
	
	// open ~/.point2play and return if it doesn't exist
	DIR* Dir = opendir(path);
	if(Dir == NULL){
        return;
	}
    
    // read first entries in ~/.point2play
	struct dirent* entry;	
	do {
		entry = readdir(Dir);
		if(entry->d_name == NULL) break;
	} while( *(entry->d_name) == '.' );
	
	GtkTreeIter iter;
		
	// go through each entry in  ~/.point2play
	while(entry != NULL)
	{	
		// create path to entry
		char* gamepath = malloc(strlen(path) + 1 + strlen(entry->d_name));
		sprintf(gamepath, "%s/%s", path, entry->d_name);
		
		// check that it's a directory
		struct stat fileinfo;
		stat(gamepath, &fileinfo);
		
		if(S_ISDIR(fileinfo.st_mode))
		{
			// Add game
			gtk_tree_store_append (gametree, &iter, NULL);
			gtk_tree_store_set(
				gametree, &iter, 
				NAME_COLUMN, entry->d_name,
				PATH_COLUMN, gamepath,
				TYPE_COLUMN, P2P_DIR_TYPE,
				-1);
			
			LoadIcon(gametree, &iter, PACKAGE_PIXMAPS_DIR"/transgaming_small.png");
			
			// open game directory
			DIR* gamedir = opendir(gamepath);
			struct dirent* gameentry;
			do 
				gameentry = readdir(gamedir);
			while( *(gameentry->d_name) == '.' );
			
			// traverse gamedir entries
			GtkTreeIter nestiter;
			while(gameentry != NULL)
			{
				// look for a .
				char* dot = strrchr(gameentry->d_name, '.');
				if(dot != NULL)
				{
					// look for .config
					if( strcmp(dot, ".config") == 0 )
					{
						// cut out .config from gameentry->d_name
						*dot = '\0';
						
						char* path = malloc(strlen(entry->d_name) + 1 + strlen(gameentry->d_name) + 1);						
						sprintf(path, "%s/%s", entry->d_name, gameentry->d_name);
						// add shortcut to tree
						gtk_tree_store_append(gametree, &nestiter, &iter);
						gtk_tree_store_set(
							gametree, &nestiter,
							NAME_COLUMN, gameentry->d_name,
							PATH_COLUMN, path,
							TYPE_COLUMN, P2P_TYPE,
							-1);
						free(path);	
						
						// open icons directory
						char* iconspath = malloc(strlen(gamepath) + 6 + 1);
						sprintf(iconspath, "%s/icons", gamepath);
						
						DIR* iconsdir = opendir(iconspath);
						struct dirent* iconsentry;
						do
							iconsentry = readdir(iconsdir);
						while(*(iconsentry->d_name) == '.');
						
						// traverse icons directory looking for shortcut.xpm
						while(iconsentry != NULL)
						{
							// if it starts with shortcut name and is followed by .
							if( strncmp(iconsentry->d_name, gameentry->d_name, strlen(gameentry->d_name)) == 0
								&& *(iconsentry->d_name + strlen(gameentry->d_name)) == '.' )
							{
								char* iconpath = malloc( strlen(iconspath) + 1 + strlen(iconsentry->d_name) + 1 );
								sprintf(iconpath, "%s/%s", iconspath, iconsentry->d_name);
								LoadIcon(gametree, &nestiter, iconpath);
								free(iconpath);
							}
							
							iconsentry = readdir(iconsdir);
						} // while iconsentry != NULL
						
						closedir(iconsdir);
						free(iconspath);
					} // if .config
				}// if .
				
				gameentry = readdir(gamedir);
			} // while gameentry != NULL
			closedir(gamedir);			
		} // if entry is dir
		
		free(gamepath);
		entry = readdir(Dir);
	}// while entry != NULL
	free(path);
	closedir(Dir);
}

void tree_TreeViewFill(GtkTreeView* TreeView, char* path)
{
    if(path == NULL)
		return;
	
	GtkTreeStore* gametree = gtk_tree_store_new(
		NUM_COLUMNS,     // duh
		GDK_TYPE_PIXBUF, // icon column
		G_TYPE_STRING,   // name column
		G_TYPE_STRING,   // path column
		G_TYPE_INT		 // type column
	);
	
    FillStore(gametree, path, NULL);
	
	char* p2ppref = prefs_GetPref("IncludeP2PGames");
	if(p2ppref == NULL)
		AddP2PEntries(gametree);
	else if(*p2ppref == 'Y')
		AddP2PEntries(gametree);
	
	free(p2ppref);
	
    gtk_tree_view_set_model(GTK_TREE_VIEW(TreeView), GTK_TREE_MODEL(gametree) );
}

char* tree_GetPathFromSelection(GtkTreeSelection* selection)
{
	GtkTreeIter iter;
	GtkTreeModel* gametree;
	char* path;
	
	if(gtk_tree_selection_get_selected(selection, &gametree, &iter)){
		gtk_tree_model_get(gametree, &iter, PATH_COLUMN, &path, -1);
		return path;
	} else
		return NULL;
}

int tree_GetTypeFromSelection(GtkTreeSelection* selection)
{
	GtkTreeIter iter;
	GtkTreeModel* gametree;
	int type;
	
	if(gtk_tree_selection_get_selected(selection, &gametree, &iter)){
		gtk_tree_model_get(gametree, &iter, TYPE_COLUMN, &type, -1);
		return type;
	}
	
	else
		return -1;
}
